==> SomatiCA_1.6.0.tar.gz: SomatiCA/R/somaticRatio.R <==
somaticRatio <- function(seg, data, method = "mle", adjust = FALSE){
	
######## Written by Mengjie Chen, Nov 2012
######## Last time modified on Jan, 2013	
	if(class(seg) != "GRanges" | class(data) != "GRanges")
	stop("Wrong input. Please check the manual.")			
	x <- as(data, "data.frame")	
	tumor.med <- median(x$tCount)
	normal.med <- median(x$tCountN)
  if(adjust == FALSE)  x$adjust.tCount <- x$tCount
	else  x$adjust.tCount <- x$tCount*normal.med/tumor.med 
  
	suppressPackageStartupMessages(require(foreach))			
	remove.outlier <- function(data, k = 0.1 ){
		quant <- quantile(data, probs = c(k, 1-k))
		flag <- data > quant[1] & data < quant[2]
		return(c(1:length(data))[flag])
	}
	i <- NULL
	results <- foreach(i=1:length(start(seg)), .combine=rbind) %do% { 		
	  sub <- x[x$start < end(seg)[i] & x$start >= start(seg)[i] & x$seqnames == as.character(seqnames(seg)[i]), ]
	  if(length(sub)==0)
		stop("Segment and data doesn't match.")	
		a1 <- as.numeric(sub$adjust.tCount)
		a2 <- as.numeric(sub$tCountN)
		aa <- intersect(remove.outlier(a1), remove.outlier(a2))
		aR <- a1/a2	
		if(method == "mean") {
			suppressPackageStartupMessages(require(sn))
			if(length(a1[aa]) > 5){			
				est.tumor.mean <- sn.mle(y = a1[aa], plot.it=F)$cp[1]
				est.normal.mean <- sn.mle(y = a2[aa], plot.it=F)$cp[1]
			}
			else{
				est.tumor.mean <- mean(a1[aa])
				est.normal.mean <- mean(a2[aa])
			}
			somaticR <- round(est.tumor.mean/est.normal.mean, 3)	
		}
		tt <- aR[aa]
		fr <- function(l){
        	sum((tt-l)^2/(tt^2+l))
        }
		mle.somaticR <- round(optimize(fr, c(0, 10))$minimum, 3)
		geometric.somaticR <- round(exp(sum(log(tt))/length(tt)), 3)  
		ratio <- switch(method, mle = mle.somaticR, mean = somaticR, 
						geometric = geometric.somaticR) 
		return(c(round(median(sub$LAF, na.rm=T), 4), round(median(sub$germLAF, na.rm=T), 4), ratio)) 			
	}
	  withratio <- GRanges(seqnames=seqnames(seg), 
	                   ranges=ranges(seg),
	                   medLAF=as.numeric(results[, 1]),
	                   medgLAF=as.numeric(results[, 2]),
                     ratio=as.numeric(results[, 3]))  
    return(withratio)   	
}

==> SomatiCA_2.2.1.tar.gz: SomatiCA/R/somaticRatio.R <==
somaticRatio <- function(seg, data, method = "mle", adjust = FALSE){
	
######## Written by Mengjie Chen, Nov 2012
######## Last time modified on Jan, 2013	
	if(class(seg) != "GRanges" | class(data) != "GRanges")
	stop("Wrong input. Please check the manual.")			
	x <- as(data, "data.frame")	
	tumor.med <- median(x$tCount)
	normal.med <- median(x$tCountN)
  if(adjust == FALSE)  x$adjust.tCount <- x$tCount
	else  x$adjust.tCount <- x$tCount*normal.med/tumor.med 
  
	suppressPackageStartupMessages(require(foreach))			
	remove.outlier <- function(data, k = 0.1 ){
		quant <- quantile(data, probs = c(k, 1-k))
		flag <- data > quant[1] & data < quant[2]
		return(c(1:length(data))[flag])
	}
	i <- NULL
	results <- foreach(i=1:length(start(seg)), .combine=rbind) %do% { 		
	  sub <- x[x$start < end(seg)[i] & x$start >= start(seg)[i] & x$seqnames == as.character(seqnames(seg)[i]), ]
	  if(length(sub)==0)
		stop("Segment and data doesn't match.")	
		a1 <- as.numeric(sub$adjust.tCount)
		a2 <- as.numeric(sub$tCountN)
		aa <- intersect(remove.outlier(a1), remove.outlier(a2))
		aR <- a1/a2	
		if(method == "mean") {
			suppressPackageStartupMessages(require(sn))
			if(length(a1[aa]) > 5){			
				est.tumor.mean <- sn.mle(y = a1[aa], plot.it=F)$cp[1]
				est.normal.mean <- sn.mle(y = a2[aa], plot.it=F)$cp[1]
			}
			else{
				est.tumor.mean <- mean(a1[aa])
				est.normal.mean <- mean(a2[aa])
			}
			somaticR <- round(est.tumor.mean/est.normal.mean, 3)	
		}
		tt <- aR[aa]
		fr <- function(l){
        	sum((tt-l)^2/(tt^2+l))
        }
		mle.somaticR <- round(optimize(fr, c(0, 10))$minimum, 3)
		geometric.somaticR <- round(exp(sum(log(tt))/length(tt)), 3)  
		ratio <- switch(method, mle = mle.somaticR, mean = somaticR, 
						geometric = geometric.somaticR) 
		return(c(round(median(sub$LAF[grep("het", sub$zygosity)], na.rm=T), 4), round(median(sub$germLAF[grep("het", sub$zygosity)], na.rm=T), 4), ratio)) 			
	}
	  withratio <- GRanges(seqnames=seqnames(seg), 
	                   ranges=ranges(seg),
	                   medLAF=as.numeric(results[, 1]),
	                   medgLAF=as.numeric(results[, 2]),
                     ratio=as.numeric(results[, 3]))  
    return(withratio)   	
}
